# -*- coding: utf8 -*-
import os
import tempfile

from django.conf import settings
from django.db import models
from django.utils.encoding import force_bytes, force_text

import pytest

from aesfield.field import AESField, EncryptedField
from aesfield.management.commands.generate_aes_keys import (
    Command, CommandError)


class AESTestModel(models.Model):
    key = AESField(max_length=255)


class TestBasic(object):

    def test_lookup(self):
        with pytest.raises(EncryptedField):
            AESTestModel.objects.filter(key='asd')

    def test_no_prefix(self):
        with pytest.raises(ValueError):
            AESField(aes_prefix='')

    def test_get_key(self, settings):
        key = 'some-super-secret-key'
        temporary_file = tempfile.NamedTemporaryFile()
        temporary_file.write(force_bytes(key))
        temporary_file.flush()

        settings.AES_KEYS = {'default': temporary_file.name}
        assert AESField().get_aes_key() == 'some-super-secret-key'

    def test_generate_fails(self):
        with pytest.raises(CommandError):
            Command().handle()

    @pytest.mark.parametrize('key', [
        'foo', '1234567890123456', '12345678901234567',
        u'ራ'.encode('utf8'),
    ])
    def test_encrypt_decrypt(self, key):
        test_model = AESTestModel()
        field = test_model._meta.get_field('key')
        assert force_text(key) == field._decrypt(field._encrypt(key))

    def test_not_encoded(self):
        test_model = AESTestModel()
        field = test_model._meta.get_field('key')
        key = u'ራ'
        assert key == field._decrypt(field._encrypt(key))

    def test_deconstruct(self):
        test_model = AESTestModel()
        field = test_model._meta.get_field('key')
        name, path, args, kwargs = field.deconstruct()
        assert kwargs['aes_method'] == 'aesfield.default'
        assert kwargs['aes_prefix'] == 'aes:'
        assert kwargs['aes_key'] == ''

    def test_reinitialize_with_deconstruct(self):
        test_model = AESTestModel()
        field = test_model._meta.get_field('key')
        name, path, args, kwargs = field.deconstruct()

        AESField(*args, **kwargs)



from importlib import import_module

from django.conf import settings
from django.db import models
from django.db.migrations.writer import SettingsReference
from django.utils.encoding import smart_bytes, smart_text

from m2secret import Secret


class EncryptedField(Exception):
    pass


class AESField(models.TextField):
    description = 'A field that uses AES encryption.'

    def __init__(self, *args, **kwargs):
        self.aes_prefix = kwargs.pop('aes_prefix', 'aes:')
        if not self.aes_prefix:
            raise ValueError('AES Prefix cannot be null.')
        self.aes_method = kwargs.pop(
            'aes_method', getattr(settings, 'AES_METHOD', 'aesfield.default'))
        self.aes_key = kwargs.pop('aes_key', '')
        super(AESField, self).__init__(*args, **kwargs)

    def deconstruct(self):
        name, path, args, kwargs = super(AESField, self).deconstruct()
        kwargs['aes_method'] = SettingsReference(self.aes_method, 'AES_METHOD')
        kwargs['aes_prefix'] = self.aes_prefix
        kwargs['aes_key'] = self.aes_key
        return name, path, args, kwargs

    def get_aes_key(self):
        result = import_module(self.aes_method).lookup(self.aes_key)
        if len(result) < 10:
            raise ValueError('Passphrase cannot be less than 10 chars.')
        return result

    def _no_lookup(self, *args, **kwargs):
        raise EncryptedField('You cannot do lookups on an encrypted field.')

    get_db_prep_lookup = get_prep_lookup = get_lookup = _no_lookup

    def from_db_value(self, value, expression, connection, context):
        if not value or not value.startswith(self.aes_prefix):
            return value
        return self._decrypt(value[len(self.aes_prefix):])

    def to_python(self, value):
        if not value or not value.startswith(self.aes_prefix):
            return value
        return self._decrypt(value[len(self.aes_prefix):])

    def get_prep_value(self, value):
        if not value:
            return value

        return self.aes_prefix + self._encrypt(value)

    def _encrypt(self, value):
        secret = Secret()
        secret.encrypt(smart_bytes(value), self.get_aes_key())
        return secret.serialize()

    def _decrypt(self, value):
        secret = Secret()
        secret.deserialize(value)
        return smart_text(secret.decrypt(self.get_aes_key()))



import os
from optparse import make_option

from django.conf import settings
from django.core.management.base import BaseCommand, CommandError


def generate_key(byte_length):
    """
    Return a true random ascii string that is byte_length long.
    The resulting key is suitable for cryptogrpahy.
    """
    if byte_length < 32:
        raise ValueError('um, %s is probably not long enough for cryptography'
                         % byte_length)
    return os.urandom(byte_length).encode('hex')


class Command(BaseCommand):
    help = 'Generate a randomized encryption encryption key'

    def add_arguments(self, parser):
        parser.add_argument(
            '--length', action='store', type=int, dest='length',
            help='Key length in bytes. Default: %(default)',
            default=128)

    def handle(self, *args, **options):
        failures = False
        for dest in settings.AES_KEYS.values():
            if os.path.exists(dest):
                failures = True

                self.stdout.write(
                    self.style.NOTICE('Not overwriting file: %s' % dest))
                continue

            with open(dest, 'wb') as fp:
                fp.write(generate_key(options['length']))

            os.chmod(dest, 0o0600)
            self.stdout.write(self.style.SUCCESS('Wrote new key: %s' % dest))

        if failures:
            raise CommandError('At least one key file already exists, '
                               'not overriding.')



from django.conf import settings


def lookup(key=None):
    """
    This is the default way to look up the keys for AES. But this is completely
    overridable if you'd like.
    """
    keys = settings.AES_KEYS
    key = 'default' if not key else key
    if key not in keys:
        raise ValueError('Key %s not found.' % key)

    with open(settings.AES_KEYS[key]) as fp:
        return fp.read()
